import re
from enum import Enum


class ArtistRoles(Enum):
    MAIN = 1
    FEATURE = 2
    REMIXER = 3
    PRODUCER = 4
    COMPOSER = 5
    CONDUCTOR = 6
    DJMIXER = 7


class TagDate:
    def __init__(self, value):
        value = str(value)  # ID3TimeStamp object sometimes comes through.
        self.year = self.date = None
        if value:
            if re.match(r'\d{4}$', value):
                self.year = int(value)
            else:
                # There may be a time value after the date... allow that and other crap.
                m = re.match(r'(\d{4}-\d{2}-\d{2})', value)
                if m:
                    self.date = m[1]
                    self.year = int(self.date[:4])


def unpack_first(value):
    if isinstance(value, list):
        return value[0] if value else None
    return value


def pack_list(value):
    return value if isinstance(value, list) else [value]



import base64

from mutagen.flac import FLAC, Picture, VCFLACDict
from mutagen.flac import error as FLACError
from mutagen.oggopus import OggOpus, OggOpusVComment
from mutagen.oggvorbis import OggVCommentDict, OggVorbis

from .base import BaseTag


class VorbisTag:
    tags_title = ['title']
    tags_version = ['version']
    tags_date = ['date', 'year']
    tags_track_number = ['tracknumber']
    tags_track_total = ['tracktotal']
    tags_disc_number = ['discnumber']
    tags_disc_total = ['disctotal']
    tags_album = ['album']
    tags_genre = ['genre']
    tags_label = ['organization', 'label', 'recordlabel']
    tags_catalog_number = ['catalognumber']
    tags_release_type = ['releasetype']
    tags_comment = ['comment']

    tags_artist_album = ['albumartist']
    tags_artist_main = ['artist']
    tags_artist_remixer = ['remixer']
    tags_artist_producer = ['producer']
    tags_artist_composer = ['composer']
    tags_artist_conductor = ['conductor']
    tags_artist_djmixer = ['djmixer']

    def create_tags_object(self):
        self.mut.add_tags()

    @property
    def _picture(self):
        if getattr(self, '__picture__', None):
            return self.__picture

        data = (self.mut.get('metadata_block_picture') or [None])[0]
        if data:
            try:
                self.__picture = Picture(base64.b64decode(data))
                return self.__picture
            except FLACError:
                pass

        return None

    @property
    def image_mime(self):
        return self._picture.mime if self._picture else None

    @property
    def image(self):
        return self._picture.data if self._picture else None


class FLACTag(VorbisTag, BaseTag):
    extensions = ('.flac',)
    mutagen_type = FLAC
    tags_type = VCFLACDict


class OggVorbisTag(VorbisTag, BaseTag):
    extensions = ('.ogg',)
    mutagen_type = OggVorbis
    tags_type = OggVCommentDict


class OggOpusTag(VorbisTag, BaseTag):
    extensions = ('.ogg',)
    mutagen_type = OggOpus
    tags_type = OggOpusVComment



import os
from pathlib import Path

from mutagen import File

from tagfiles._common import ArtistRoles, TagDate  # noqa
from tagfiles.errors import NonIntegerTag, UnsupportedFileType  # noqa

from . import id3  # noqa
from . import mp4  # noqa
from . import vorbis  # noqa
from .base import BaseTag


def TagFile(filepath):
    if isinstance(filepath, Path):
        filepath = str(filepath.resolve())
    mut = File(filepath)
    ext = os.path.splitext(filepath)[1]
    try:
        allowed_exts = BaseTag.get_allowed_extensions()
        for subcls in allowed_exts[ext]:
            if isinstance(mut, subcls.mutagen_type):
                return subcls(filepath, mut)
    except KeyError:
        pass
    raise UnsupportedFileType



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tagfiles']

package_data = \
{'': ['*']}

install_requires = \
['mutagen>=1.42,<2.0']

setup_kwargs = {
    'name': 'tagfiles',
    'version': '0.3.3',
    'description': 'A tagging interface for multiple audio formats and metadata containers.',
    'long_description': '# tagfiles\n\n[![CI](https://img.shields.io/github/workflow/status/azuline/tagfiles/CI)](https://github.com/azuline/tagfiles/actions)\n[![codecov](https://img.shields.io/codecov/c/github/azuline/tagfiles?token=VF37TOZ5HJ)](https://codecov.io/gh/azuline/tagfiles)\n[![Pypi](https://img.shields.io/pypi/v/tagfiles.svg)](https://pypi.python.org/pypi/tagfiles)\n[![Pyversions](https://img.shields.io/pypi/pyversions/tagfiles.svg)](https://pypi.python.org/pypi/tagfiles)\n\nA tagging interface for multiple audio formats and metadata containers.\n\nThe supported audio codecs and containers are:\n\n- FLAC in FLAC container\n- MP3 in MP3 container\n- AAC in MP4 container\n- Vorbis in Ogg container\n- Opus in Ogg container\n\nTag mappings are derived from https://picard.musicbrainz.org/docs/mappings/ .\n\n## Usage\n\n```python\n>>> from tagfiles import TagFile, ArtistRoles\n>>> from pprint import pprint\n>>>\n>>> tf = TagFile(\'/home/azuline/02. No Captain.m4a\')\n>>> print(tf.title)\nNo Captain\n>>> pprint(tf.artist)\n{<ArtistRoles.MAIN: 1>: [\'Lane 8\'],\n <ArtistRoles.FEATURE: 2>: [\'Poliça\'],\n <ArtistRoles.REMIXER: 3>: [],\n <ArtistRoles.PRODUCER: 4>: [],\n <ArtistRoles.COMPOSER: 5>: [\'\'],\n <ArtistRoles.CONDUCTOR: 6>: [],\n <ArtistRoles.DJMIXER: 7>: []}\n>>> print(tf.artist[ArtistRoles.MAIN])\n[\'Lane 8\']\n>>> print(tf.date.year)\n2015\n>>> print(tf.date.date)\n2015-01-19\n>>>\n>>> tf.date = \'2018-01-19\'  # Fixing the date!\n>>> print(tf.date.date)\n2018-01-19\n>>> print(tf.date.year)\n2018\n>>> tf.save()\n>>>\n>>> tf = TagFile(\'/home/azuline/music.txt\')\nTraceback (most recent call last):\n  File "<stdin>", line 1, in <module>\n  File "/home/azuline/devel/tagfiles/tagfiles/__init__.py", line 27, in TagFile\n    raise UnsupportedFileType\ntagfiles.errors.UnsupportedFileType\n```\n\nThe TagFile function takes a filepath as a parameter and returns the class\nwhich corresponds to its container. If an unsupported filetype is passed in,\nthe `UnsupportedFileType` error is raised. Each class presents the same metadata\ninterface, which have the following attributes:\n\n```python\ntitle: str\nversion: str\nalbum: str\nartist_album: List[str]\ncatalog_number: str\nrelease_type: str\ncomment: str\ndate.year: int\ndate.date: str\ntrack_number: str\ntrack_total: str\ndisc_number: str\ndisc_total: str\ngenre: List[str]\nlabel: str\nartist = {\n  ArtistRoles.MAIN: List[str]\n  ArtistRoles.FEATURE: List[str]\n  ArtistRoles.REMIXER: List[str]\n  ArtistRoles.PRODUCER: List[str]\n  ArtistRoles.COMPOSER: List[str]\n  ArtistRoles.CONDUCTOR: List[str]\n  ArtistRoles.DJMIXER: List[str]\n}\nimage_mime: str\nimage: bytes\n```\n\nFields can be edited by setting new values to the attributes of the TagFile.\nTo edit the date, which is special, assign a string in the format of `%Y-%m-%d`\nor `%Y` to the `date` attribute. To save the changes made to the tags, call the\n`save()` method.\n\n_Note: Image field is not currently editable._\n',
    'author': 'azuline',
    'author_email': 'azuline@riseup.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/azuline/tagfiles',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
